﻿/*
 * 
 *          DremDOS
 *          Mouse Driver Class (MouseDriver.cs)
 * 
 *          Contains the mouse driver
 *
 */

using System;
using System.Collections.Generic;
using System.Text;
using Sys = Cosmos.System;
using System;
using System.Collections.Generic;
using System.Text;
using Cosmos.HAL.Drivers.PCI.Video;
using Cosmos.System.Graphics;
using System.Runtime.InteropServices;
using System.Diagnostics;
using Cosmos.System;
using System.Drawing;
using Cosmos.Core.IOGroup;
using Console = System.Console;
using DremDOS.Core.Terminal;
using DremDOS.Core.Graphics;
using DremDOS.Core.Drivers;
using DremDOS.Core.Sound;

namespace DremDOS.Core.Drivers
{
    public class MouseDriver
    {
        private bool MouseHasMoved = false;
        public static ushort X
        {
            get
            {
                return (ushort)Sys.MouseManager.X;
            }
        }
        public static ushort Y
        {
            get
            {
                return (ushort)Sys.MouseManager.Y;
            }
        }
        public uint ScreenWidth
        {
            get
            {
                return Sys.MouseManager.ScreenWidth;
            }
            set
            {
                Sys.MouseManager.ScreenWidth = value;
            }
        }
        public uint ScreenHeight
        {
            get
            {
                return Sys.MouseManager.ScreenHeight;
            }
            set
            {
                Sys.MouseManager.ScreenHeight = value;
            }
        }
        public MouseDriver(uint screenWidth, uint screenHeight)
        {
            Sys.MouseManager.ScreenWidth = screenWidth;
            Sys.MouseManager.ScreenHeight = screenHeight;
        }

        public static int LastDrawX { get; private set; } = 0;

        public static int LastDrawY { get; private set; } = 0;

        /*private readonly uint[][] LastDrawPosCols = new uint[20][]
        {
            new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20], new uint[20],
        };*/
        private readonly Color[][] LastDrawPosCols = new Color[20][]
        {
            new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20], new Color[20],
        };
        private readonly Color[][] MouseCols = new Color[20][]
        {
            new Color[20] { Color.Black, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.Black, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.White, Color.White, Color.Black, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.White, Color.Black, Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Black, Color.White, Color.White, Color.White, Color.White, Color.White, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Black, Color.White, Color.White, Color.Black, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent },
            new Color[20] { Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Black, Color.Black, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent, Color.Transparent }
        };
        
        public void Draw(Canvas canvas)
        {
            if (LastDrawX != X || LastDrawY != Y)
            {
                DoDraw(LastDrawX, LastDrawY, canvas, LastDrawPosCols, false);

                for (ushort x = 0; x < 20; x++) for (ushort y = 0; y < 20; y++) LastDrawPosCols[y][x] = canvas.GetPointColor((ushort)(X + x), (ushort)(Y + y));

                LastDrawX = X;
                LastDrawY = Y;
            }

            //DoDraw(X, Y, canvas, MouseCols, false);
        }

        private void DoDraw(int X, int Y, Canvas canvas, Color[][] mouseCol, bool allowBlack)
        {
            for (ushort x = 0; x < 20; x++) for (ushort y = 0; y < 20; y++) if (allowBlack || mouseCol[y][x] != Color.Transparent) canvas.DrawPoint(new Pen(mouseCol[y][x]), new Cosmos.System.Graphics.Point((ushort)(X + x), (ushort)(Y + y)));
            //canvas.DrawACSIIString(new Cosmos.System.Graphics.Pen(Color.White), "]", X, Y);
        }

        public static int GetX()
        {
            return X;
        }

        public static int GetY()
        {
            return Y;
        }
        public static bool GetLeftClick()
        {
            if (Cosmos.System.MouseManager.MouseState == Cosmos.System.MouseState.Left)
            {
                return true;
            }
            return false;
        }

        public static bool GetRightClick()
        {
            if (Cosmos.System.MouseManager.MouseState == Cosmos.System.MouseState.Right)
            {
                return true;
            }
            return false;
        }
        public bool HasMoved()
        {
            if(!MouseHasMoved && (GetX() != 0 || GetY() != 0)) { MouseHasMoved = true; }
            return MouseHasMoved;
        }
    }
}