/*
 * 
 *          DremDOS
 *          Sound Class (Sound.cs)
 * 
 *          Contains Beep() function and more complex sounds
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Sys = Cosmos.System;

namespace DremDOS.Core.Sound
{
    public class Sound
    {
        public static void Beep(string[] arguments)
        {
            // Initialize variables
            int frequency = 0;
            int time = 0;
            // Test if there are enough arguments
            if (arguments.Length >= 2)
            {
                if (int.TryParse(arguments[0], out frequency) && int.TryParse(arguments[1], out time))
                {
                    // Checks if the frequency is within the frequencies allowed by C#
                    if (frequency >= 37 && frequency <= 32767)
                    {
                        Console.Beep(frequency, time);
                    }
                    else // If one of the arguments can't be casted to an int, display the following error.
                    {
                        Console.Beep(1000, 100);
                        Console.Write("Error: Frequency must be between 37 and 32767Hz\n");
                    }
                }
            }
            else // If there aren't enough arguments, do the default beep.
            {
                Console.Beep(1000, 1000);
            }
        }
        public static void Shutdown()
        {
            Console.Beep(1975, 200);
            Console.Beep(1318, 200);
            Console.Beep(880, 200);
            Console.Beep(987, 300);
        }

        public static void Startup()
        {
            Console.Beep(4000, 100);
            Console.Beep(8000, 100);
        }

        public static void Dance()
        {
            Console.Beep(587, 100); // D
            Console.Beep(659, 100); // E
            Console.Beep(698, 100); // F
            Console.Beep(659, 100); // E
            Console.Beep(587, 100); // D
            Console.Beep(587, 100); // D
            Console.Beep(440, 100); // A
            Console.Beep(440, 100); // A
            Console.Beep(659, 100); // E
            Console.Beep(659, 100); // E
            Console.Beep(659, 100); // E
            Console.Beep(587, 100); // D
            Console.Beep(523, 150); // C
            Console.Beep(587, 200); // D
            Console.Beep(30000, 200); // N/A
            Console.Beep(587, 200); // D
            Console.Beep(30000, 200); // N/A
            Console.Beep(523, 200); // C
            Console.Beep(587, 200); // D
        }
    }
}